@@ -34,7 +34,7 @@
 #include <86box/hdc.h>
 #include <86box/hdc_ide.h>
 #include <86box/hdc_ide_sff8038i.h>
-#include <86box/zip.h>
+#include <86box/rdisk.h>
 #include <86box/mo.h>
 
 typedef struct cmd640_t {
@@ -45,7 +45,7 @@ typedef struct cmd640_t {
     uint8_t  pci;
     uint8_t  irq_state;
     uint8_t  pci_slot;
-    uint8_t  pad0;
+    uint8_t  force_on;
     uint8_t  regs[256];
     uint32_t local;
     int      irq_mode[2];
@@ -143,7 +143,7 @@ cmd640_ide_handlers(cmd640_t *dev)
         ide_set_base(0, main);
         ide_set_side(0, side);
 
-        if (dev->regs[0x04] & 0x01)
+        if ((dev->regs[0x04] & 0x01) || dev->force_on)
             ide_pri_enable();
     }
 
@@ -161,7 +161,7 @@ cmd640_ide_handlers(cmd640_t *dev)
         ide_set_base(1, main);
         ide_set_side(1, side);
 
-        if ((dev->regs[0x04] & 0x01) && (dev->regs[0x51] & 0x08))
+        if (((dev->regs[0x04] & 0x01) || dev->force_on) && (dev->regs[0x51] & 0x08))
             ide_sec_enable();
     }
 }
@@ -417,10 +417,10 @@ cmd640_reset(void *priv)
             (cdrom[i].ide_channel <= max_channel) && cdrom[i].priv)
             scsi_cdrom_reset((scsi_common_t *) cdrom[i].priv);
     }
-    for (i = 0; i < ZIP_NUM; i++) {
-        if ((zip_drives[i].bus_type == ZIP_BUS_ATAPI) && (zip_drives[i].ide_channel >= min_channel) &&
-            (zip_drives[i].ide_channel <= max_channel) && zip_drives[i].priv)
-            zip_reset((scsi_common_t *) zip_drives[i].priv);
+    for (i = 0; i < RDISK_NUM; i++) {
+        if ((rdisk_drives[i].bus_type == RDISK_BUS_ATAPI) && (rdisk_drives[i].ide_channel >= min_channel) &&
+            (rdisk_drives[i].ide_channel <= max_channel) && rdisk_drives[i].priv)
+            rdisk_reset((scsi_common_t *) rdisk_drives[i].priv);
     }
     for (i = 0; i < MO_NUM; i++) {
         if ((mo_drives[i].bus_type == MO_BUS_ATAPI) && (mo_drives[i].ide_channel >= min_channel) &&
@@ -504,8 +504,7 @@ cmd640_close(void *priv)
 static void *
 cmd640_init(const device_t *info)
 {
-    cmd640_t *dev = (cmd640_t *) malloc(sizeof(cmd640_t));
-    memset(dev, 0x00, sizeof(cmd640_t));
+    cmd640_t *dev = (cmd640_t *) calloc(1, sizeof(cmd640_t));
 
     dev->id = next_id | 0x60;
 
@@ -513,6 +512,7 @@ cmd640_init(const device_t *info)
     dev->local = info->local;
 
     dev->channels = ((info->local & 0x60000) >> 17) & 0x03;
+    dev->force_on = !!(info->local & 0x100000);
 
     if (info->flags & DEVICE_PCI) {
         device_add(&ide_pci_2ch_device);
@@ -563,7 +563,7 @@ const device_t ide_cmd640_vlb_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -577,7 +577,7 @@ const device_t ide_cmd640_vlb_178_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -591,7 +591,7 @@ const device_t ide_cmd640_vlb_pri_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -605,7 +605,7 @@ const device_t ide_cmd640_vlb_pri_178_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -619,7 +619,7 @@ const device_t ide_cmd640_vlb_sec_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -633,7 +633,7 @@ const device_t ide_cmd640_vlb_sec_178_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -647,7 +647,7 @@ const device_t ide_cmd640_pci_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -661,35 +661,35 @@ const device_t ide_cmd640_pci_legacy_only_device = {
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
 };
 
 const device_t ide_cmd640_pci_single_channel_device = {
-    .name          = "CMD PCI-0640B PCI",
+    .name          = "CMD PCI-0640B PCI (Single Channel)",
     .internal_name = "ide_cmd640_pci_single_channel",
     .flags         = DEVICE_PCI,
     .local         = 0x2000a,
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
 };
 
 const device_t ide_cmd640_pci_single_channel_sec_device = {
-    .name          = "CMD PCI-0640B PCI",
+    .name          = "CMD PCI-0640B PCI (Single Channel, Secondary)",
     .internal_name = "ide_cmd640_pci_single_channel_sec",
     .flags         = DEVICE_PCI,
     .local         = 0x4000a,
     .init          = cmd640_init,
     .close         = cmd640_close,
     .reset         = cmd640_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
