@@ -79,6 +79,8 @@ int floppyrate[4];
 
 int fdc_current[FDC_MAX] = { 0, 0 };
 
+volatile int fdcinited = 0;
+
 #ifdef ENABLE_FDC_LOG
 int fdc_do_log = ENABLE_FDC_LOG;
 
@@ -103,15 +105,20 @@ typedef const struct {
 
 static fdc_cards_t fdc_cards[] = {
     // clang-format off
-    { &device_none        },
-    { &device_internal    },
-    { &fdc_xt_device      },
-    { &fdc_at_device      },
-    { &fdc_b215_device    },
-    { &fdc_pii151b_device },
-    { &fdc_pii158b_device },
-    { &fdc_monster_device },
-    { NULL                }
+    { &device_none               },
+    { &device_internal           },
+    { &fdc_xt_device             },
+    { &fdc_at_device             },
+    { &fdc_b215_device           },
+    { &fdc_pii151b_device        },
+    { &fdc_pii158b_device        },
+    { &fdc_compaticard_i_device  },
+    { &fdc_compaticard_ii_device },
+#if 0
+    { &fdc_compaticard_iv_device },
+#endif
+    { &fdc_monster_device        },
+    { NULL                       }
     // clang-format on
 };
 
@@ -177,15 +184,21 @@ fdc_ctrl_reset(void *priv)
 {
     fdc_t *fdc = (fdc_t *) priv;
 
-    fdc->stat = 0x80;
+    fdc->stat             = 0x80;
     fdc->pnum = fdc->ptot = 0;
     fdc->st0              = 0;
-    fdc->lock             = 0;
     fdc->head             = 0;
     fdc->step             = 0;
     fdc->power_down       = 0;
-    if (!(fdc->flags & FDC_FLAG_AT))
-        fdc->rate = 2;
+
+    if (!fdc->lock && !fdc->fifointest) {
+        fdc->fifo  = 0;
+        fdc->tfifo = 1;
+
+        fifo_reset(fdc->fifo_p);
+        fifo_set_len(fdc->fifo_p, fdc->tfifo + 1);
+        fifo_set_trigger_len(fdc->fifo_p, fdc->tfifo + 1);
+    }
 }
 
 sector_id_t
@@ -273,6 +286,15 @@ fdc_is_mfm(fdc_t *fdc)
     return fdc->mfm ? 1 : 0;
 }
 
+int
+fdc_is_dma(fdc_t *fdc)
+{
+    if ((fdc->flags & FDC_FLAG_PCJR) || !fdc->dma)
+        return 0;
+    else
+        return 1;
+}
+
 void
 fdc_request_next_sector_id(fdc_t *fdc)
 {
@@ -313,14 +335,18 @@ fdc_int(fdc_t *fdc, int set_fintr)
 {
     int ienable = 0;
 
-    if (!(fdc->flags & FDC_FLAG_PCJR))
+    if (fdc->flags & FDC_FLAG_PS2_MCA)
+        ienable = 1;
+    else if (!(fdc->flags & FDC_FLAG_PCJR))
         ienable = !!(fdc->dor & 8);
 
-    if (ienable)
-        picint(1 << fdc->irq);
+    if (ienable) {
+        if (fdc->irq != 0xff)
+            picint(1 << fdc->irq);
 
-    if (set_fintr)
-        fdc->fintr = 1;
+        if (set_fintr)
+            fdc->fintr = 1;
+    }
     fdc_log("fdc_int(%i): fdc->fintr = %i\n", set_fintr, fdc->fintr);
 }
 
@@ -333,7 +359,7 @@ fdc_watchdog_poll(void *priv)
     if (fdc->watchdog_count)
         timer_advance_u64(&fdc->watchdog_timer, 1000 * TIMER_USEC);
     else {
-        if (fdc->dor & 0x20)
+        if ((fdc->dor & 0x20) && (fdc->irq != 0xff))
             picint(1 << fdc->irq);
     }
 }
@@ -363,6 +389,15 @@ fdc_set_power_down(fdc_t *fdc, uint8_t power_down)
     fdc->power_down = power_down;
 }
 
+void
+fdc_toggle_flag(fdc_t *fdc, int flag, int on)
+{
+    if (on)
+        fdc->flags |= flag;
+    else
+        fdc->flags &= ~flag;
+}
+
 void
 fdc_update_max_track(fdc_t *fdc, int max_track)
 {
@@ -390,6 +425,58 @@ fdc_update_rwc(fdc_t *fdc, int drive, int rwc)
     fdc_rate(fdc, drive);
 }
 
+uint8_t
+fdc_get_media_id(fdc_t *fdc, int id)
+{
+    uint8_t ret = fdc->media_id & (1 << id);
+
+    return ret;
+}
+
+void
+fdc_set_media_id(fdc_t *fdc, int id, int set)
+{
+    fdc->media_id = (fdc->media_id & ~(1 << id)) | (set << id);
+}
+
+void
+fdc_set_flags(fdc_t *fdc, int flags)
+{
+    fdc->flags |= flags;
+}
+
+void
+fdc_clear_flags(fdc_t *fdc, int flags)
+{
+    fdc->flags &= ~flags;
+}
+
+void
+fdc_set_fdd_changed(int drive, int changed)
+{
+    if (changed)
+        fdd_changed[drive] = 1;
+}
+
+uint8_t
+fdc_get_fdd_changed(int drive)
+{
+    uint8_t ret = !!fdd_changed[drive];
+
+    return ret;
+}
+
+uint8_t
+fdc_get_shadow(fdc_t *fdc)
+{
+    uint8_t ret = (fdc->rate & 0x03) |
+                  ((fdc->pretrk & 0x07) << 2) |
+                  (fdc->power_down ? 0x40 : 0x00) |
+                  ((fdc_read(0x03f2, fdc) & 0x04) ? 0x80 : 0x00);
+
+    return ret;
+}
+
 int
 fdc_get_boot_drive(fdc_t *fdc)
 {
@@ -441,9 +528,11 @@ fdc_update_drv2en(fdc_t *fdc, int drv2en)
 void
 fdc_update_rate(fdc_t *fdc, int drive)
 {
-    if (((fdc->rwc[drive] == 1) || (fdc->rwc[drive] == 2)) && fdc->enh_mode)
+    if (((fdc->rwc[drive] == 1) || (fdc->rwc[drive] == 2)) &&
+        fdc->enh_mode && !(fdc->flags & FDC_FLAG_SMC661))
         fdc->bit_rate = 500;
-    else if ((fdc->rwc[drive] == 3) && fdc->enh_mode)
+    else if ((fdc->rwc[drive] == 3) && fdc->enh_mode &&
+             !(fdc->flags & FDC_FLAG_SMC661))
         fdc->bit_rate = 250;
     else  switch (fdc->rate) {
         default:
@@ -507,7 +596,7 @@ fdc_get_bitcell_period(fdc_t *fdc)
 static int
 fdc_get_densel(fdc_t *fdc, int drive)
 {
-    if (fdc->enh_mode) {
+    if (fdc->enh_mode && !(fdc->flags & FDC_FLAG_SMC661)) {
         switch (fdc->rwc[drive]) {
             case 1:
             case 3:
@@ -610,11 +699,16 @@ fdc_io_command_phase1(fdc_t *fdc, int out)
         }
     }
 
-    ui_sb_update_icon(SB_FLOPPY | real_drive(fdc, fdc->drive), 1);
-    fdc->stat = out ? 0x10 : 0x50;
-    if ((fdc->flags & FDC_FLAG_PCJR) || !fdc->dma)
-        fdc->stat |= 0x20;
+    if (fdc->processed_cmd == 0x05 || fdc->processed_cmd == 0x09)
+        ui_sb_update_icon_write(SB_FLOPPY | real_drive(fdc, fdc->drive), 1);
     else
+        ui_sb_update_icon(SB_FLOPPY | real_drive(fdc, fdc->drive), 1);
+    fdc->stat = out ? 0x10 : 0x50;
+    if ((fdc->flags & FDC_FLAG_PCJR) || !fdc->dma) {
+        fdc->stat |= 0x20;
+        if (out)
+            fdc->stat |= 0x80;
+    } else
         dma_set_drq(fdc->dma_ch, 1);
 }
 
@@ -652,13 +746,33 @@ fdc_sis(fdc_t *fdc)
     fdc->paramstogo = 2;
 }
 
+static void
+fdc_soft_reset(fdc_t *fdc)
+{
+    if (fdc->power_down) {
+        timer_set_delay_u64(&fdc->timer, 1000 * TIMER_USEC);
+        fdc->interrupt = -5;
+    } else {
+        timer_set_delay_u64(&fdc->timer, 8 * TIMER_USEC);
+        fdc->interrupt = -1;
+
+        fdc->perp &= 0xfc;
+
+        for (int i = 0; i < FDD_NUM; i++) {
+            ui_sb_update_icon(SB_FLOPPY | i, 0);
+            ui_sb_update_icon_write(SB_FLOPPY | i, 0);
+        }
+
+        fdc_ctrl_reset(fdc);
+   }
+}
+
 static void
 fdc_write(uint16_t addr, uint8_t val, void *priv)
 {
     fdc_t *fdc = (fdc_t *) priv;
 
     int drive;
-    int i;
     int drive_num;
 
     fdc_log("Write FDC %04X %02X\n", addr, val);
@@ -681,8 +795,8 @@ fdc_write(uint16_t addr, uint8_t val, void *priv)
                     timer_set_delay_u64(&fdc->timer, 8 * TIMER_USEC);
                     fdc->interrupt = -1;
                     ui_sb_update_icon(SB_FLOPPY | 0, 0);
+                    ui_sb_update_icon_write(SB_FLOPPY | 0, 0);
                     fdc_ctrl_reset(fdc);
-                    fdd_changed[0] = 1;
                 }
                 if (!fdd_get_flags(0))
                     val &= 0xfe;
@@ -690,33 +804,27 @@ fdc_write(uint16_t addr, uint8_t val, void *priv)
                 fdc->st0 &= ~0x07;
                 fdc->st0 |= (fdd_get_head(0) ? 4 : 0);
             } else {
-                if (!(val & 8) && (fdc->dor & 8)) {
-                    fdc->tc = 1;
-                    fdc_int(fdc, 1);
+                /*
+                   Writing this bit to logic "1" will enable the DRQ,
+                   nDACK, TC and FINTR outputs. This bit being a
+                   logic "0" will disable the nDACK and TC inputs, and
+                   hold the DRQ and FINTR outputs in a high
+                   impedance state.
+                 */
+                if (!(val & 8) && (fdc->dor & 8) && !(fdc->flags & FDC_FLAG_PS2_MCA)) {
+                    fdc->tc    = 1;
+                    fdc->fintr = 0;
+                    picintc(1 << fdc->irq);
                 }
                 if (!(val & 4)) {
                     fdd_stop(real_drive(fdc, val & 3));
                     fdc->stat = 0x00;
                     fdc->pnum = fdc->ptot = 0;
                 }
-                if ((val & 4) && !(fdc->dor & 4)) {
-                    if (fdc->power_down) {
-                        timer_set_delay_u64(&fdc->timer, 1000 * TIMER_USEC);
-                        fdc->interrupt = -5;
-                    } else {
-                        timer_set_delay_u64(&fdc->timer, 8 * TIMER_USEC);
-                        fdc->interrupt = -1;
-
-                        fdc->perp &= 0xfc;
-
-                        for (i = 0; i < FDD_NUM; i++)
-                            ui_sb_update_icon(SB_FLOPPY | i, 0);
-
-                        fdc_ctrl_reset(fdc);
-                    }
-                }
+                if ((val & 4) && !(fdc->dor & 4))
+                    fdc_soft_reset(fdc);
                 /* We can now simplify this since each motor now spins separately. */
-                for (i = 0; i < FDD_NUM; i++) {
+                for (int i = 0; i < FDD_NUM; i++) {
                     drive_num = real_drive(fdc, i);
                     if ((!fdd_get_flags(drive_num)) || (drive_num >= FDD_NUM))
                         val &= ~(0x10 << drive_num);
@@ -731,36 +839,50 @@ fdc_write(uint16_t addr, uint8_t val, void *priv)
             return;
         case 3: /* TDR */
             if (fdc->enh_mode) {
-                drive = real_drive(fdc, fdc->dor & 3);
-                fdc_update_rwc(fdc, drive, (val & 0x30) >> 4);
+                if (fdc->flags & FDC_FLAG_SMC661) {
+                    fdc_set_swap(fdc, !!(val & 0x20));
+                    fdc_update_densel_force(fdc, (val & 0x18) >> 3);
+                } else {
+                    drive = real_drive(fdc, fdc->dor & 3);
+                    fdc_update_rwc(fdc, drive, (val & 0x30) >> 4);
+                }
+            }
+            /* Bit 2: FIFO test mode (PS/55 5550-S,T only. Undocumented) 
+               The Power-on Self Test of PS/55 writes and verifies 8 bytes of FIFO buffer through I/O 3F5h.
+               If it fails, then floppy drives will be treated as DD drives. */
+            if (fdc->flags & FDC_FLAG_PS2_MCA) {
+                if (val & 0x04) {
+                    fdc->tfifo      = 8;
+                    fdc->fifointest = 1;
+                } else {
+                    fdc->tfifo      = 1;
+                    fdc->fifointest = 0;
+                }
+                fifo_reset(fdc->fifo_p);
+                fifo_set_len(fdc->fifo_p, fdc->tfifo + 1);
+                fifo_set_trigger_len(fdc->fifo_p, fdc->tfifo + 1);
             }
             return;
-        case 4:
+        case 4: /* DSR */
             if (!(fdc->flags & FDC_FLAG_NO_DSR_RESET)) {
                 if (!(val & 0x80)) {
                     timer_set_delay_u64(&fdc->timer, 8 * TIMER_USEC);
                     fdc->interrupt = -6;
                 }
-                if (fdc->power_down || ((val & 0x80) && !(fdc->dsr & 0x80))) {
-                    if (fdc->power_down) {
-                        timer_set_delay_u64(&fdc->timer, 1000 * TIMER_USEC);
-                        fdc->interrupt = -5;
-                    } else {
-                        timer_set_delay_u64(&fdc->timer, 8 * TIMER_USEC);
-                        fdc->interrupt = -1;
-
-                        fdc->perp &= 0xfc;
-
-                        for (i = 0; i < FDD_NUM; i++)
-                            ui_sb_update_icon(SB_FLOPPY | i, 0);
-
-                        fdc_ctrl_reset(fdc);
-                    }
-                }
+                if (fdc->power_down || ((val & 0x80) && !(fdc->dsr & 0x80)))
+                    fdc_soft_reset(fdc);
             }
             fdc->dsr = val;
             return;
         case 5: /*Command register*/
+            if (fdc->fifointest) {
+                /* Write FIFO buffer in the test mode (PS/55) */
+                fdc_log("FIFO buffer position = %X\n", ((fifo_t *) fdc->fifo_p)->end);
+                fifo_write(val, fdc->fifo_p);
+                if (fifo_get_full(fdc->fifo_p))
+                    fdc->stat &= ~0x80;
+                break;
+            }
             if ((fdc->stat & 0xf0) == 0xb0) {
                 if ((fdc->flags & FDC_FLAG_PCJR) || !fdc->fifo) {
                     fdc->dat = val;
@@ -1209,7 +1331,7 @@ fdc_write(uint16_t addr, uint8_t val, void *priv)
             if (!(fdc->flags & FDC_FLAG_TOSHIBA) && !(fdc->flags & FDC_FLAG_AT) && !(fdc->flags & FDC_FLAG_UMC))
                 return;
             fdc->rate = val & 0x03;
-            if (fdc->flags & FDC_FLAG_PS1)
+            if (fdc->flags & FDC_FLAG_PS2)
                 fdc->noprec = !!(val & 0x04);
             return;
 
@@ -1229,23 +1351,43 @@ fdc_read(uint16_t addr, void *priv)
 
     if (!fdc->power_down || ((addr & 7) == 2))  switch (addr & 7) {
         case 0: /* STA */
-            if (fdc->flags & FDC_FLAG_PS1) {
+            if (fdc->flags & FDC_FLAG_PS2) {
                 drive = real_drive(fdc, fdc->dor & 3);
                 ret   = 0x00;
                 /* TODO:
                         Bit 2: INDEX (best return always 0 as it goes by very fast)
                 */
-                if (fdc->seek_dir) /* nDIRECTION */
+                if (fdc->seek_dir)                 /* nDIRECTION */
                     ret |= 0x01;
-                if (writeprot[drive]) /* WRITEPROT */
+                if (writeprot[drive])              /* WRITEPROT */
                     ret |= 0x02;
-                if (!fdd_get_head(drive)) /* nHDSEL */
+                if (!fdd_get_head(drive))          /* nHDSEL */
                     ret |= 0x08;
-                if (fdd_track0(drive)) /* TRK0 */
+                if (fdd_track0(drive))             /* TRK0 */
                     ret |= 0x10;
-                if (fdc->step) /* STEP */
+                if (fdc->step)                     /* STEP */
                     ret |= 0x20;
-                if (dma_get_drq(fdc->dma_ch)) /* DRQ */
+                if (dma_get_drq(fdc->dma_ch))      /* DRQ */
+                    ret |= 0x40;
+                if (fdc->fintr || fdc->reset_stat) /* INTR */
+                    ret |= 0x80;
+            } else if (fdc->flags & FDC_FLAG_PS2_MCA) {
+                drive = real_drive(fdc, fdc->dor & 3);
+                ret   = 0x04;
+                /* TODO:
+                        Bit 2: nINDEX (best return always 1 as it goes by very fast)
+                */
+                if (!fdc->seek_dir)                /* DIRECTION */
+                    ret |= 0x01;
+                if (!writeprot[drive])             /* nWRITEPROT */
+                    ret |= 0x02;
+                if (fdd_get_head(drive))           /* HDSEL */
+                    ret |= 0x08;
+                if (!fdd_track0(drive))            /* nTRK0 */
+                    ret |= 0x10;
+                if (fdc->step)                     /* STEP */
+                    ret |= 0x20;
+                if (!fdd_get_type(1))              /* -Drive 2 Installed */
                     ret |= 0x40;
                 if (fdc->fintr || fdc->reset_stat) /* INTR */
                     ret |= 0x80;
@@ -1253,14 +1395,12 @@ fdc_read(uint16_t addr, void *priv)
                 ret = 0xff;
             break;
         case 1: /* STB */
-            if (fdc->flags & FDC_FLAG_PS1) {
+            if (fdc->flags & FDC_FLAG_PS2) {
                 drive = real_drive(fdc, fdc->dor & 3);
                 ret   = 0x00;
-                /* -Drive 2 Installed */
-                if (!fdd_get_type(1))
+                if (!fdd_get_type(1))              /* -Drive 2 Installed */
                     ret |= 0x80;
-                /* -Drive Select 1,0 */
-                switch (drive) {
+                switch (drive) {                   /* -Drive Select 1,0 */
                     case 0:
                         ret |= 0x43;
                         break;
@@ -1277,6 +1417,11 @@ fdc_read(uint16_t addr, void *priv)
                     default:
                         break;
                 }
+            } else if (fdc->flags & FDC_FLAG_PS2_MCA) {
+                drive = real_drive(fdc, fdc->dor & 3);
+                ret   = 0xc0;
+                ret  |= (fdc->dor & 0x01) << 5;    /* Drive Select 0 */
+                ret  |= (fdc->dor & 0x30) >> 4;    /* Motor Select 1, 0 */
             } else {
                 if (is486 || !fdc->enable_3f1)
                     ret = 0xff;
@@ -1285,19 +1430,12 @@ fdc_read(uint16_t addr, void *priv)
                         drive = real_drive(fdc, fdc->dor & 1);
                         ret   = !fdd_is_dd(drive) ? ((fdc->dor & 1) ? 2 : 1) : 0;
                     } else {
-                        ret = 0x70;
-
+                        /* TODO: What is this and what is it used for?
+                                 It's almost identical to the PS/2 MCA mode. */
                         drive = real_drive(fdc, fdc->dor & 3);
-
-                        if (drive)
-                            ret &= ~0x40;
-                        else
-                            ret &= ~0x20;
-
-                        if (fdc->dor & 0x10)
-                            ret |= 1;
-                        if (fdc->dor & 0x20)
-                            ret |= 2;
+                        ret   = 0x70;
+                        ret  &= ~(drive ? 0x40 : 0x20);
+                        ret  |= (fdc->dor & 0x30) >> 4;    /* Motor Select 1, 0 */
                     }
                 }
             }
@@ -1307,7 +1445,8 @@ fdc_read(uint16_t addr, void *priv)
             break;
         case 3:
             drive = real_drive(fdc, fdc->dor & 3);
-            if (fdc->flags & FDC_FLAG_PS1) {
+            /* TODO: FDC_FLAG_PS2_TDR? */
+            if ((fdc->flags & FDC_FLAG_PS2) || (fdc->flags & FDC_FLAG_PS2_MCA)) {
                 /* PS/1 Model 2121 seems return drive type in port
                  * 0x3f3, despite the 82077AA fdc_t not implementing
                  * this. This is presumably implemented outside the
@@ -1326,15 +1465,23 @@ fdc_read(uint16_t addr, void *priv)
                     ret = 0x10;
                 else
                     ret = 0x00;
+                /* PS/55 POST throws an error and halt if ret = 1 or 2, somehow. */
             } else if (!fdc->enh_mode)
                 ret = 0x20;
+            else if (fdc->flags & FDC_FLAG_SMC661)
+                ret = (fdc->densel_force << 3) | ((!!fdc->swap) << 5) | (fdc->media_id << 6);
             else
-                ret = fdc->rwc[drive] << 4;
+                ret = (fdc->rwc[drive] << 4) | (fdc->media_id << 6);
             break;
         case 4: /*Status*/
             ret = fdc->stat;
             break;
         case 5: /*Data*/
+            if (fdc->fifointest) {
+                /* Read FIFO buffer in the test mode (PS/55) */
+                ret = fifo_read(fdc->fifo_p);
+                break;
+            }
             if ((fdc->stat & 0xf0) == 0xf0) {
                 fdc->stat &= ~0x80;
                 if ((fdc->flags & FDC_FLAG_PCJR) || !fdc->fifo) {
@@ -1369,7 +1516,7 @@ fdc_read(uint16_t addr, void *priv)
         case 7: /*Disk change*/
             drive = real_drive(fdc, fdc->dor & 3);
 
-            if (fdc->flags & FDC_FLAG_PS1) {
+            if (fdc->flags & FDC_FLAG_PS2) {
                 if (fdc->dor & (0x10 << drive)) {
                     ret = (fdd_changed[drive] || drive_empty[drive]) ? 0x00 : 0x80;
                     ret |= (fdc->dor & 0x08);
@@ -1377,6 +1524,14 @@ fdc_read(uint16_t addr, void *priv)
                     ret |= (fdc->rate & 0x03);
                 } else
                     ret = 0x00;
+            } else if (fdc->flags & FDC_FLAG_PS2_MCA) {
+                if (fdc->dor & (0x10 << drive)) {
+                    ret = (fdd_changed[drive] || drive_empty[drive]) ? 0x80 : 0x00;
+                    ret |= ((fdc->rate & 0x03) << 1);
+                    ret |= fdc_get_densel(fdc, drive);
+                    ret |= 0x78;
+                } else
+                    ret = 0xf9;
             } else {
                 if (fdc->dor & (0x10 << drive)) {
                     if ((drive == 1) && (fdc->flags & FDC_FLAG_TOSHIBA))
@@ -1399,7 +1554,7 @@ fdc_read(uint16_t addr, void *priv)
             fdc->step = 0;
             break;
         default:
-            ret = 0xFF;
+            ret = 0xff;
     }
     fdc_log("[%04X:%08X] Read FDC %04X %02X [%i:%02X]\n", CS, cpu_state.pc, addr, ret, drive, fdc->dor & (0x10 << drive));
     return ret;
@@ -1409,7 +1564,7 @@ static void
 fdc_poll_common_finish(fdc_t *fdc, int compare, int st5)
 {
     fdc_int(fdc, 1);
-    if (!(fdc->flags & FDC_FLAG_PS1))
+    if (!(fdc->flags & FDC_FLAG_FINTR))
         fdc->fintr = 0;
     fdc->stat = 0xD0;
     fdc->st0 = fdc->res[4] = (fdd_get_head(real_drive(fdc, fdc->drive)) ? 4 : 0) | fdc->rw_drive;
@@ -1455,6 +1610,7 @@ fdc_poll_common_finish(fdc_t *fdc, int compare, int st5)
     fdc->res[10] = fdc->params[4];
     fdc_log("Read/write finish (%02X %02X %02X %02X %02X %02X %02X)\n", fdc->res[4], fdc->res[5], fdc->res[6], fdc->res[7], fdc->res[8], fdc->res[9], fdc->res[10]);
     ui_sb_update_icon(SB_FLOPPY | real_drive(fdc, fdc->drive), 0);
+    ui_sb_update_icon_write(SB_FLOPPY | real_drive(fdc, fdc->drive), 0);
     fdc->paramstogo = 7;
     dma_set_drq(fdc->dma_ch, 0);
 }
@@ -1498,8 +1654,10 @@ fdc_callback(void *priv)
         case -5: /*Reset in power down mode */
             fdc->perp &= 0xfc;
 
-            for (uint8_t i = 0; i < FDD_NUM; i++)
+            for (uint8_t i = 0; i < FDD_NUM; i++) {
                 ui_sb_update_icon(SB_FLOPPY | i, 0);
+                ui_sb_update_icon_write(SB_FLOPPY | i, 0);
+            }
 
             fdc_ctrl_reset(fdc);
 
@@ -1556,8 +1714,8 @@ fdc_callback(void *priv)
         case 0x06: /* Read data */
         case 0x0c: /* Read deleted data */
         case 0x11: /* Scan equal */
+        case 0x16: /* Verify */
         case 0x19: /* Scan low or equal */
-        case 0x1c: /* Verify */
         case 0x1d: /* Scan high or equal */
             if ((fdc->interrupt == 0x11) || (fdc->interrupt == 0x19) || (fdc->interrupt == 0x1D))
                 compare = 1;
@@ -1647,7 +1805,10 @@ fdc_callback(void *priv)
                 fdc->sector++;
             else if (fdc->params[5] == 0)
                 fdc->sector++;
-            ui_sb_update_icon(SB_FLOPPY | real_drive(fdc, fdc->drive), 1);
+            if (fdc->interrupt == 0x05 || fdc->interrupt == 0x09)
+                ui_sb_update_icon_write(SB_FLOPPY | real_drive(fdc, fdc->drive), 1);
+            else
+                ui_sb_update_icon(SB_FLOPPY | real_drive(fdc, fdc->drive), 1);
             switch (fdc->interrupt) {
                 case 5:
                 case 9:
@@ -1710,7 +1871,7 @@ fdc_callback(void *priv)
             } else {
                 fdc->interrupt = -2;
                 fdc_int(fdc, 1);
-                if (!(fdc->flags & FDC_FLAG_PS1))
+                if (!(fdc->flags & FDC_FLAG_FINTR))
                     fdc->fintr = 0;
                 fdc->stat = 0xD0;
                 fdc->st0 = fdc->res[4] = (fdd_get_head(real_drive(fdc, fdc->drive)) ? 4 : 0) | fdc->drive;
@@ -1802,7 +1963,7 @@ fdc_error(fdc_t *fdc, int st5, int st6)
     timer_disable(&fdc->timer);
 
     fdc_int(fdc, 1);
-    if (!(fdc->flags & FDC_FLAG_PS1))
+    if (!(fdc->flags & FDC_FLAG_FINTR))
         fdc->fintr = 0;
     fdc->stat = 0xD0;
     fdc->st0 = fdc->res[4] = 0x40 | (fdd_get_head(real_drive(fdc, fdc->drive)) ? 4 : 0) | fdc->rw_drive;
@@ -1838,6 +1999,7 @@ fdc_error(fdc_t *fdc, int st5, int st6)
             break;
     }
     ui_sb_update_icon(SB_FLOPPY | real_drive(fdc, fdc->drive), 0);
+    ui_sb_update_icon_write(SB_FLOPPY | real_drive(fdc, fdc->drive), 0);
     fdc->paramstogo = 7;
 }
 
@@ -2143,9 +2305,13 @@ fdc_set_base(fdc_t *fdc, int base)
 {
     int super_io = (fdc->flags & FDC_FLAG_SUPERIO);
 
+    if (base == 0x0000) {
+        fdc->base_address = base;
+        return;
+    }
+
     if (fdc->flags & FDC_FLAG_NSC) {
-        io_sethandler(base + 2, 0x0001, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
-        io_sethandler(base + 4, 0x0002, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
+        io_sethandler(base + 2, 0x0004, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
         io_sethandler(base + 7, 0x0001, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
     } else {
         if ((fdc->flags & FDC_FLAG_AT) || (fdc->flags & FDC_FLAG_AMSTRAD)) {
@@ -2174,10 +2340,12 @@ fdc_remove(fdc_t *fdc)
 {
     int super_io = (fdc->flags & FDC_FLAG_SUPERIO);
 
+    if (fdc->base_address == 0x0000)
+        return;
+
     fdc_log("FDC Removed (%04X)\n", fdc->base_address);
     if (fdc->flags & FDC_FLAG_NSC) {
-        io_removehandler(fdc->base_address + 2, 0x0001, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
-        io_removehandler(fdc->base_address + 4, 0x0002, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
+        io_removehandler(fdc->base_address + 2, 0x0004, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
         io_removehandler(fdc->base_address + 7, 0x0001, fdc_read, NULL, NULL, fdc_write, NULL, NULL, fdc);
     } else {
         if ((fdc->flags & FDC_FLAG_AT) || (fdc->flags & FDC_FLAG_AMSTRAD)) {
@@ -2211,7 +2379,7 @@ fdc_reset(void *priv)
     fdc->enable_3f1 = 1;
 
     fdc_update_enh_mode(fdc, 0);
-    if (fdc->flags & FDC_FLAG_PS1)
+    if (fdc->flags & FDC_FLAG_DENSEL_INVERT)
         fdc_update_densel_polarity(fdc, 0);
     else
         fdc_update_densel_polarity(fdc, 1);
@@ -2223,15 +2391,27 @@ fdc_reset(void *priv)
     fdc_update_rwc(fdc, 1, default_rwc);
     fdc_update_rwc(fdc, 2, default_rwc);
     fdc_update_rwc(fdc, 3, default_rwc);
-    fdc_update_drvrate(fdc, 0, 0);
-    fdc_update_drvrate(fdc, 1, 0);
-    fdc_update_drvrate(fdc, 2, 0);
-    fdc_update_drvrate(fdc, 3, 0);
+    /*
+       The OKI IF386SX natively supports the Japanese 1.25 MB floppy format,
+       since it can read such images just fine, it also attempts to use data
+       rate 01 on a 3.5" MB drive (which is the only kind it can physically
+       take, anyway), and rate 01 on a 3.5" MB drive is usually used by 3-mode
+       drives to switch to 360 RPM. Hence why I'm switching DRVDEN to 1, so
+       rate 01 becomes 500 kbps, so on a 3-mode 3.5" drive, 1.25 MB floppies
+       can be read. The side effect is that to read 5.25" 360k drives, you
+       need to use a dual-RPM 5.25" drive - but hey, that finally gets those
+       drives some usage as well.
+     */
+    fdc_update_drvrate(fdc, 0, !strcmp(machine_get_internal_name(),  "if386sx"));
+    fdc_update_drvrate(fdc, 1, !strcmp(machine_get_internal_name(),  "if386sx"));
+    fdc_update_drvrate(fdc, 2, !strcmp(machine_get_internal_name(),  "if386sx"));
+    fdc_update_drvrate(fdc, 3, !strcmp(machine_get_internal_name(),  "if386sx"));
     fdc_update_drv2en(fdc, 1);
     fdc_update_rates(fdc);
 
     fdc->fifo  = 0;
     fdc->tfifo = 1;
+    fdc->fifointest = 0;
 
     if (fdc->flags & FDC_FLAG_PCJR) {
         fdc->dma        = 0;
@@ -2255,8 +2435,13 @@ fdc_reset(void *priv)
     fdc->swwp          = 0;
     fdc->disable_write = 0;
 
+    fdc->lock          = 0;
+
     fdc_ctrl_reset(fdc);
 
+    if (!(fdc->flags & FDC_FLAG_AT))
+        fdc->rate = 2;
+
     fdc->max_track = (fdc->flags & FDC_FLAG_MORE_TRACKS) ? 85 : 79;
 
     fdc_remove(fdc);
@@ -2271,10 +2456,14 @@ fdc_reset(void *priv)
 
     current_drive = 0;
 
-    for (uint8_t i = 0; i < FDD_NUM; i++)
+    for (uint8_t i = 0; i < FDD_NUM; i++) {
         ui_sb_update_icon(SB_FLOPPY | i, 0);
+        ui_sb_update_icon_write(SB_FLOPPY | i, 0);
+    }
 
     fdc->power_down = 0;
+
+    fdc->media_id   = 0;
 }
 
 static void
@@ -2288,14 +2477,15 @@ fdc_close(void *priv)
 
     fifo_close(fdc->fifo_p);
 
+    fdcinited = 0;
+
     free(fdc);
 }
 
 static void *
 fdc_init(const device_t *info)
 {
-    fdc_t *fdc = (fdc_t *) malloc(sizeof(fdc_t));
-    memset(fdc, 0, sizeof(fdc_t));
+    fdc_t *fdc = (fdc_t *) calloc(1, sizeof(fdc_t));
 
     fdc->flags = info->local;
 
@@ -2334,6 +2524,8 @@ fdc_init(const device_t *info)
 
     fdc_reset(fdc);
 
+    fdcinited = 1;
+
     return fdc;
 }
 
@@ -2351,7 +2543,7 @@ const device_t fdc_xt_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2365,7 +2557,7 @@ const device_t fdc_xt_sec_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2379,7 +2571,7 @@ const device_t fdc_xt_ter_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2393,7 +2585,7 @@ const device_t fdc_xt_qua_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2407,7 +2599,7 @@ const device_t fdc_xt_t1x00_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2421,7 +2613,7 @@ const device_t fdc_xt_amstrad_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2435,7 +2627,21 @@ const device_t fdc_xt_tandy_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
+    .speed_changed = NULL,
+    .force_redraw  = NULL,
+    .config        = NULL
+};
+
+const device_t fdc_xt_umc_um8398_device = {
+    .name          = "PC/XT Floppy Drive Controller (UMC UM8398)",
+    .internal_name = "fdc_xt_umc_um8398",
+    .flags         = 0,
+    .local         = FDC_FLAG_UMC,
+    .init          = fdc_init,
+    .close         = fdc_close,
+    .reset         = fdc_reset,
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2449,7 +2655,7 @@ const device_t fdc_pcjr_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2463,7 +2669,7 @@ const device_t fdc_at_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2477,7 +2683,7 @@ const device_t fdc_at_sec_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2491,7 +2697,7 @@ const device_t fdc_at_ter_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2505,7 +2711,7 @@ const device_t fdc_at_qua_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2519,35 +2725,21 @@ const device_t fdc_at_actlow_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
 };
 
-const device_t fdc_at_ps1_device = {
-    .name          = "PC/AT Floppy Drive Controller (PS/1, PS/2 ISA)",
-    .internal_name = "fdc_at_ps1",
+const device_t fdc_at_smc_661_device = {
+    .name          = "PC/AT Floppy Drive Controller (SM(s)C FDC37C661/2)",
+    .internal_name = "fdc_at_smc",
     .flags         = 0,
-    .local         = FDC_FLAG_DISKCHG_ACTLOW | FDC_FLAG_AT | FDC_FLAG_PS1,
+    .local         = FDC_FLAG_AT | FDC_FLAG_SUPERIO | FDC_FLAG_SMC661,
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
-    .speed_changed = NULL,
-    .force_redraw  = NULL,
-    .config        = NULL
-};
-
-const device_t fdc_at_ps1_2121_device = {
-    .name          = "PC/AT Floppy Drive Controller (PS/1, PS/2 ISA)",
-    .internal_name = "fdc_at_ps1",
-    .flags         = 0,
-    .local         = FDC_FLAG_NO_DSR_RESET | FDC_FLAG_DISKCHG_ACTLOW | FDC_FLAG_AT | FDC_FLAG_PS1,
-    .init          = fdc_init,
-    .close         = fdc_close,
-    .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2561,7 +2753,7 @@ const device_t fdc_at_smc_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2575,7 +2767,7 @@ const device_t fdc_at_ali_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2589,7 +2781,7 @@ const device_t fdc_at_winbond_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -2603,35 +2795,51 @@ const device_t fdc_at_nsc_device = {
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
 };
 
-const device_t fdc_dp8473_device = {
-    .name          = "NS DP8473 Floppy Drive Controller",
-    .internal_name = "fdc_dp8473",
+const device_t fdc_at_nsc_dp8473_device = {
+    .name          = "PC/AT Floppy Drive Controller (NSC DP8473)",
+    .internal_name = "fdc_at_nsc_dp8473",
     .flags         = 0,
     .local         = FDC_FLAG_AT | FDC_FLAG_NEC | FDC_FLAG_NO_DSR_RESET,
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
 };
 
-const device_t fdc_um8398_device = {
-    .name          = "UMC UM8398 Floppy Drive Controller",
-    .internal_name = "fdc_um8398",
+const device_t fdc_ps2_device = {
+    .name          = "PS/2 Model 25/30 Floppy Drive Controller",
+    .internal_name = "fdc_ps2",
     .flags         = 0,
-    .local         = FDC_FLAG_UMC,
+    .local         = FDC_FLAG_FINTR | FDC_FLAG_DENSEL_INVERT | FDC_FLAG_NO_DSR_RESET | FDC_FLAG_DISKCHG_ACTLOW |
+                     FDC_FLAG_AT | FDC_FLAG_PS2,
     .init          = fdc_init,
     .close         = fdc_close,
     .reset         = fdc_reset,
-    { .available = NULL },
+    .available     = NULL,
+    .speed_changed = NULL,
+    .force_redraw  = NULL,
+    .config        = NULL
+};
+
+const device_t fdc_ps2_mca_device = {
+    .name          = "PS/2 MCA Floppy Drive Controller",
+    .internal_name = "fdc_ps2_mca",
+    .flags         = 0,
+    .local         = FDC_FLAG_FINTR | FDC_FLAG_DENSEL_INVERT | FDC_FLAG_NO_DSR_RESET | FDC_FLAG_AT |
+                     FDC_FLAG_PS2_MCA,
+    .init          = fdc_init,
+    .close         = fdc_close,
+    .reset         = fdc_reset,
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
