@@ -54,7 +54,9 @@
 #include <stdlib.h>
 #include <wchar.h>
 #include <time.h>
+#ifndef _MSC_VER
 #include <sys/time.h>
+#endif
 #include <stdbool.h>
 #define HAVE_STDARG_H
 #include <86box/86box.h>
@@ -75,40 +77,53 @@
 #    include <winsock2.h>
 #endif
 
-static const device_t *net_cards[] = {
-    &device_none,
-    &device_internal,
-    &threec501_device,
-    &threec503_device,
-    &pcnet_am79c960_device,
-    &pcnet_am79c961_device,
-    &de220p_device,
-    &ne1000_compat_device,
-    &ne2000_compat_device,
-    &ne2000_compat_8bit_device,
-    &ne1000_device,
-    &ne2000_device,
-    &pcnet_am79c960_eb_device,
-    &rtl8019as_device,
-    &wd8003e_device,
-    &wd8003eb_device,
-    &wd8013ebt_device,
-    &plip_device,
-    &ethernext_mc_device,
-    &wd8003eta_device,
-    &wd8003ea_device,
-    &wd8013epa_device,
-    &pcnet_am79c973_device,
-    &pcnet_am79c970a_device,
-    &dec_tulip_device,
-    &rtl8029as_device,
-    &rtl8139c_plus_device,
-    &dec_tulip_21140_device,
-    &dec_tulip_21140_vpc_device,
-    &dec_tulip_21040_device,
-    &pcnet_am79c960_vlb_device,
-    &modem_device,
-    NULL
+typedef struct {
+    const device_t *device;
+} NETWORK_CARD;
+
+static const NETWORK_CARD net_cards[] = {
+    // clang-format off
+    { &device_none                },
+    { &device_internal            },
+    /* ISA */
+    { &threec501_device           },
+    { &threec503_device           },
+    { &ne1000_compat_device       },
+    { &ne2000_compat_8bit_device  },
+    { &ne1000_device              },
+    { &wd8003e_device             },
+    { &wd8003eb_device            },
+    { &wd8013ebt_device           },
+    /* COM */
+    { &modem_device               },
+    /* LPT */
+    { &plip_device                },
+    /* ISA16 */
+    { &pcnet_am79c960_device      },
+    { &pcnet_am79c961_device      },
+    { &de220p_device              },
+    { &ne2000_compat_device       },
+    { &ne2000_device              },
+    { &pcnet_am79c960_eb_device   },
+    { &rtl8019as_pnp_device       },
+    /* MCA */
+    { &ethernext_mc_device        },
+    { &wd8003eta_device           },
+    { &wd8003ea_device            },
+    { &wd8013epa_device           },
+    /* VLB */
+    { &pcnet_am79c960_vlb_device  },
+    /* PCI */
+    { &pcnet_am79c973_device      },
+    { &pcnet_am79c970a_device     },
+    { &dec_tulip_21140_device     },
+    { &dec_tulip_21040_device     },
+    { &dec_tulip_device           },
+    { &dec_tulip_21140_vpc_device },
+    { &rtl8029as_device           },
+    { &rtl8139c_plus_device       },
+    { NULL                        }
+    // clang-format on
 };
 
 netcard_conf_t net_cards_conf[NET_CARD_MAX];
@@ -424,7 +439,8 @@ network_rx_queue(void *priv)
     bool activity = rx_bytes || tx_bytes;
     bool led_on   = card->led_timer & 0x80000000;
     if ((activity && !led_on) || (card->led_timer & 0x7fffffff) >= 150000) {
-        ui_sb_update_icon(SB_NETWORK | card->card_num, activity);
+        ui_sb_update_icon(SB_NETWORK | card->card_num, !!(rx_bytes));
+        ui_sb_update_icon_write(SB_NETWORK | card->card_num, !!(tx_bytes));
         card->led_timer = 0 | (activity << 31);
     }
 
@@ -481,6 +497,19 @@ network_attach(void *card_drv, uint8_t *mac, NETRXCB rx, NETSETLINKSTATE set_lin
             card->host_drv.priv = card->host_drv.init(card, mac, net_cards_conf[net_card_current].host_dev_name, net_drv_error);
             break;
 #endif
+#ifdef HAS_TAP
+        case NET_TYPE_TAP:
+            card->host_drv      = net_tap_drv;
+            card->host_drv.priv = card->host_drv.init(card, mac, net_cards_conf[net_card_current].host_dev_name, net_drv_error);
+            break;
+#endif
+#ifdef USE_NETSWITCH
+        case NET_TYPE_NMSWITCH:
+        case NET_TYPE_NRSWITCH:
+            card->host_drv      = net_netswitch_drv;
+            card->host_drv.priv = card->host_drv.init(card, mac, &net_cards_conf[net_card_current], net_drv_error);
+            break;
+#endif /* USE_NETSWITCH */
         default:
             card->host_drv.priv = NULL;
             break;
@@ -492,8 +521,16 @@ network_attach(void *card_drv, uint8_t *mac, NETRXCB rx, NETSETLINKSTATE set_lin
     if (!card->host_drv.priv) {
 
         if(net_cards_conf[net_card_current].net_type != NET_TYPE_NONE) {
+#ifdef USE_NETSWITCH
+            // FIXME: Hardcoded during dev
+            // FIXME: Remove when done!
+            if((net_cards_conf[net_card_current].net_type == NET_TYPE_NMSWITCH) ||
+                (net_cards_conf[net_card_current].net_type == NET_TYPE_NRSWITCH))
+                fatal("%s", net_drv_error);
+#endif /* USE_NETSWITCH */
+
             // We're here because of a failure
-            swprintf(tempmsg, sizeof_w(tempmsg), L"%ls:<br /><br />%s<br /><br />%ls", plat_get_string(STRING_NET_ERROR), net_drv_error, plat_get_string(STRING_NET_ERROR_DESC));
+            swprintf(tempmsg, sizeof_w(tempmsg), L"%ls:\n\n%s\n\n%ls", plat_get_string(STRING_NET_ERROR), net_drv_error, plat_get_string(STRING_NET_ERROR_DESC));
             ui_msgbox(MBX_ERROR, tempmsg);
             net_cards_conf[net_card_current].net_type = NET_TYPE_NONE;
         }
@@ -569,6 +606,7 @@ void
 network_reset(void)
 {
     ui_sb_update_icon(SB_NETWORK, 0);
+    ui_sb_update_icon_write(SB_NETWORK, 0);
 
 #ifdef ENABLE_NETWORK_LOG
     network_dump_mutex = thread_create_mutex();
@@ -581,7 +619,7 @@ network_reset(void)
 
         net_card_current = i;
         if (net_cards_conf[i].device_num > NET_INTERNAL)
-            device_add_inst(net_cards[net_cards_conf[i].device_num], i + 1);
+            device_add_inst(net_cards[net_cards_conf[i].device_num].device, i + 1);
     }
 }
 
@@ -635,6 +673,43 @@ network_rx_put(netcard_t *card, uint8_t *bufp, int len)
     return ret;
 }
 
+int
+network_rx_on_tx_popv(netcard_t *card, netpkt_t *pkt_vec, int vec_size)
+{
+    int pkt_count = 0;
+
+    netqueue_t *queue = &card->queues[NET_QUEUE_RX_ON_TX];
+    for (int i = 0; i < vec_size; i++) {
+        if (!network_queue_get_swap(queue, pkt_vec))
+            break;
+        network_dump_packet(pkt_vec);
+        pkt_count++;
+        pkt_vec++;
+    }
+
+    return pkt_count;
+}
+
+int
+network_rx_on_tx_put(netcard_t *card, uint8_t *bufp, int len)
+{
+    int ret = 0;
+
+    ret = network_queue_put(&card->queues[NET_QUEUE_RX_ON_TX], bufp, len);
+
+    return ret;
+}
+
+int
+network_rx_on_tx_put_pkt(netcard_t *card, netpkt_t *pkt)
+{
+    int ret = 0;
+
+    ret = network_queue_put_swap(&card->queues[NET_QUEUE_RX_ON_TX], pkt);
+
+    return ret;
+}
+
 int
 network_rx_put_pkt(netcard_t *card, netpkt_t *pkt)
 {
@@ -712,8 +787,8 @@ network_available(void)
 int
 network_card_available(int card)
 {
-    if (net_cards[card])
-        return (device_available(net_cards[card]));
+    if (net_cards[card].device)
+        return (device_available(net_cards[card].device));
 
     return 1;
 }
@@ -722,24 +797,24 @@ network_card_available(int card)
 const device_t *
 network_card_getdevice(int card)
 {
-    return (net_cards[card]);
+    return (net_cards[card].device);
 }
 
 /* UI */
 int
 network_card_has_config(int card)
 {
-    if (!net_cards[card])
+    if (!net_cards[card].device)
         return 0;
 
-    return (device_has_config(net_cards[card]) ? 1 : 0);
+    return (device_has_config(net_cards[card].device) ? 1 : 0);
 }
 
 /* UI */
 const char *
 network_card_get_internal_name(int card)
 {
-    return device_get_internal_name(net_cards[card]);
+    return device_get_internal_name(net_cards[card].device);
 }
 
 /* UI */
@@ -748,8 +823,8 @@ network_card_get_from_internal_name(char *s)
 {
     int c = 0;
 
-    while (net_cards[c] != NULL) {
-        if (!strcmp(net_cards[c]->internal_name, s))
+    while (net_cards[c].device != NULL) {
+        if (!strcmp(net_cards[c].device->internal_name, s))
             return c;
         c++;
     }
