@@ -419,6 +419,9 @@ pci_trc_reset(uint8_t val)
         mem_a20_recalc();
 
         flushmmucache();
+
+        if (is_p6)
+            mem_zero();
     }
 
 #ifdef USE_DYNAREC
@@ -790,7 +793,7 @@ pci_add_card(uint8_t add_type, uint8_t (*read)(int func, int addr, void *priv),
     if (next_pci_card < PCI_CARDS_NUM) {
         dev = &pci_card_descs[next_pci_card];
 
-        dev->type = add_type | PCI_ADD_STRICT;
+        dev->type  = add_type | PCI_ADD_STRICT;
         dev->read  = read;
         dev->write = write;
         dev->priv  = priv;
@@ -852,10 +855,10 @@ pci_register_card(int pci_card)
 
 /* Add an instance of the PCI bridge. */
 void
-pci_add_bridge(uint8_t agp, uint8_t (*read)(int func, int addr, void *priv), void (*write)(int func, int addr, uint8_t val, void *priv), void *priv, uint8_t *slot)
+pci_add_bridge(uint8_t add_type, uint8_t (*read)(int func, int addr, void *priv), void (*write)(int func, int addr, uint8_t val, void *priv), void *priv, uint8_t *slot)
 {
     pci_card_t *card;
-    uint8_t bridge_slot = agp ? pci_find_slot(PCI_ADD_AGPBRIDGE, 0xff) : last_normal_pci_card_id;
+    uint8_t bridge_slot = (add_type == PCI_ADD_NORMAL) ? last_normal_pci_card_id : pci_find_slot(add_type, 0xff);
 
     if (bridge_slot != PCI_CARD_INVALID) {
         card = &pci_cards[bridge_slot];
@@ -885,7 +888,7 @@ pci_register_cards(void)
             type = pci_card_descs[i].type;
             slot = pci_card_descs[i].slot;
 #endif
-            normal = (pci_card_descs[i].type == PCI_CARD_NORMAL);
+            normal = ((pci_card_descs[i].type & ~PCI_ADD_STRICT) == PCI_CARD_NORMAL);
 
             /* If this is a normal card, increase the next normal card index. */
             if (normal)
