@@ -18,6 +18,11 @@
  *          Copyright 2021-2022 Cacodemon345
  *          Copyright 2021-2022 Teemu Korhonen
  */
+
+#ifdef __HAIKU__
+#include <OS.h>
+#endif
+
 #include <cstdio>
 
 #include <mutex>
@@ -38,6 +43,7 @@
 #include <QTimer>
 #include <QProcess>
 #include <QRegularExpression>
+#include <QKeySequence>
 
 #include <QLibrary>
 #include <QElapsedTimer>
@@ -49,11 +55,19 @@
 #include "qt_progsettings.hpp"
 #include "qt_util.hpp"
 
+#ifndef Q_OS_WINDOWS
+#    include <signal.h>
+#endif
+
 #ifdef Q_OS_UNIX
 #    include <pthread.h>
 #    include <sys/mman.h>
 #endif
 
+#ifdef Q_OS_OPENBSD
+#    include <pthread_np.h>
+#endif
+
 #if 0
 static QByteArray buf;
 #endif
@@ -74,13 +88,13 @@ public:
     CharPointer &operator=(const QByteArray &ba)
     {
         if (s > 0) {
+            // If the size is known, copy up to s - 1 bytes
+            // and null-terminate the string.
             strncpy(b, ba.data(), s - 1);
-            b[s] = 0;
-        } else {
-            // if we haven't been told the length of b, just assume enough
-            // because we didn't get it from emulator code
+            b[s - 1] = 0;
+        } else if (ba.size() > 0) {
+            // If the size is unknown, copy the whole QByteArray
             strcpy(b, ba.data());
-            b[ba.size()] = 0;
         }
         return *this;
     }
@@ -127,7 +141,6 @@ int          update_icons    = 1;
 int          kbd_req_capture = 0;
 int          hide_status_bar = 0;
 int          hide_tool_bar   = 0;
-uint32_t     lang_id = 0x0409, lang_sys = 0x0409; // Multilangual UI variables, for now all set to LCID of en-US
 
 int
 stricmp(const char *s1, const char *s2)
@@ -152,6 +165,7 @@ strnicmp(const char *s1, const char *s2, size_t n)
 void
 do_start(void)
 {
+    //
 }
 
 void
@@ -163,14 +177,28 @@ do_stop(void)
 #endif
 }
 
+extern bool acp_utf8;
 void
 plat_get_exe_name(char *s, int size)
 {
+#ifdef Q_OS_WINDOWS
+    wchar_t *temp;
+
+    if (acp_utf8)
+        GetModuleFileNameA(NULL, s, size);
+    else {
+        temp = (wchar_t*)calloc(size, sizeof(wchar_t));
+        GetModuleFileNameW(NULL, temp, size);
+        c16stombs(s, (uint16_t*)temp, size);
+        free(temp);
+    }
+#else
     QByteArray exepath_temp = QCoreApplication::applicationDirPath().toLocal8Bit();
 
     memcpy(s, exepath_temp.data(), std::min((qsizetype) exepath_temp.size(), (qsizetype) size));
 
     path_slash(s);
+#endif
 }
 
 uint32_t
@@ -227,7 +255,12 @@ plat_getcwd(char *bufp, int max)
 {
 #ifdef __APPLE__
     /* Working directory for .app bundles is undefined. */
+#ifdef USE_EXE_PATH
     strncpy(bufp, exe_path, max);
+#else
+    CharPointer(bufp, max) = QDir::homePath().toUtf8();
+    path_append_filename(bufp, bufp, "Library/86Box");
+#endif
 #else
     CharPointer(bufp, max) = QDir::currentPath().toUtf8();
 #endif
@@ -322,9 +355,27 @@ path_get_slash(char *path)
 void
 path_append_filename(char *dest, const char *s1, const char *s2)
 {
-    strcpy(dest, s1);
-    path_slash(dest);
-    strcat(dest, s2);
+    size_t dest_size = 260;
+    size_t len;
+
+    if (!dest || !s1 || !s2)
+        return;
+
+    if (dest != s1)
+        snprintf(dest, dest_size, "%s", s1);
+
+    len = strlen(dest);
+
+    if (len > 0 && dest[len - 1] != '/' && dest[len - 1] != '\\') {
+        if (len + 1 < dest_size) {
+            dest[len++] = '/';
+            dest[len] = '\0';
+        }
+    }
+
+    if (len < dest_size - 1) {
+        strncat(dest, s2, dest_size - len - 1);
+    }
 }
 
 void
@@ -356,6 +407,10 @@ plat_mmap(size_t size, uint8_t executable)
 #elif defined Q_OS_UNIX
 #    if defined Q_OS_DARWIN && defined MAP_JIT
     void *ret = mmap(0, size, PROT_READ | PROT_WRITE | (executable ? PROT_EXEC : 0), MAP_ANON | MAP_PRIVATE | (executable ? MAP_JIT : 0), -1, 0);
+#    elif defined(PROT_MPROTECT)
+    void *ret = mmap(0, size, PROT_MPROTECT(PROT_READ | PROT_WRITE | (executable ? PROT_EXEC : 0)), MAP_ANON | MAP_PRIVATE, -1, 0);
+    if (ret)
+        mprotect(ret, size, PROT_READ | PROT_WRITE | (executable ? PROT_EXEC : 0));
 #    else
     void *ret = mmap(0, size, PROT_READ | PROT_WRITE | (executable ? PROT_EXEC : 0), MAP_ANON | MAP_PRIVATE, -1, 0);
 #    endif
@@ -373,6 +428,7 @@ plat_munmap(void *ptr, size_t size)
 #endif
 }
 
+extern bool cpu_thread_running;
 void
 plat_pause(int p)
 {
@@ -380,7 +436,13 @@ plat_pause(int p)
     wchar_t        title[1024];
     wchar_t        paused_msg[512];
 
+    if (!cpu_thread_running && p == 1) {
+        p = 2;
+    }
+
     if ((!!p) == dopause) {
+        QTimer::singleShot(0, main_window, &MainWindow::updateUiPauseState);
+
 #ifdef Q_OS_WINDOWS
         if (source_hwnd)
             PostMessage((HWND) (uintptr_t) source_hwnd, WM_SENDSTATUS, (WPARAM) !!p, (LPARAM) (HWND) main_window->winId());
@@ -433,57 +495,18 @@ plat_power_off(void)
     QTimer::singleShot(0, (const QWidget *) main_window, &QMainWindow::close);
 }
 
-extern "C++" {
-QMap<uint32_t, QPair<QString, QString>> ProgSettings::lcid_langcode = {
-    { 0x0403, { "ca-ES", "Catalan (Spain)" }         },
-    { 0x0804, { "zh-CN", "Chinese (Simplified)" }    },
-    { 0x0404, { "zh-TW", "Chinese (Traditional)" }   },
-    { 0x041A, { "hr-HR", "Croatian (Croatia)" }      },
-    { 0x0405, { "cs-CZ", "Czech (Czech Republic)" }  },
-    { 0x0407, { "de-DE", "German (Germany)" }        },
-    { 0x0809, { "en-GB", "English (United Kingdom)" }},
-    { 0x0409, { "en-US", "English (United States)" } },
-    { 0x040B, { "fi-FI", "Finnish (Finland)" }       },
-    { 0x040C, { "fr-FR", "French (France)" }         },
-    { 0x040E, { "hu-HU", "Hungarian (Hungary)" }     },
-    { 0x0410, { "it-IT", "Italian (Italy)" }         },
-    { 0x0411, { "ja-JP", "Japanese (Japan)" }        },
-    { 0x0412, { "ko-KR", "Korean (Korea)" }          },
-    { 0x0415, { "pl-PL", "Polish (Poland)" }         },
-    { 0x0416, { "pt-BR", "Portuguese (Brazil)" }     },
-    { 0x0816, { "pt-PT", "Portuguese (Portugal)" }   },
-    { 0x0419, { "ru-RU", "Russian (Russia)" }        },
-    { 0x041B, { "sk-SK", "Slovak (Slovakia)" }       },
-    { 0x0424, { "sl-SI", "Slovenian (Slovenia)" }    },
-    { 0x0C0A, { "es-ES", "Spanish (Spain, Modern Sort)" } },
-    { 0x041F, { "tr-TR", "Turkish (Turkey)" }        },
-    { 0x0422, { "uk-UA", "Ukrainian (Ukraine)" }     },
-    { 0x042A, { "vi-VN", "Vietnamese (Vietnam)" }    },
-    { 0xFFFF, { "system", "(System Default)" }       },
-};
-}
-
-/* Sets up the program language before initialization. */
-uint32_t
+/* Converts the language code string to a numeric language ID */
+int
 plat_language_code(char *langcode)
 {
-    for (auto &curKey : ProgSettings::lcid_langcode.keys()) {
-        if (ProgSettings::lcid_langcode[curKey].first == langcode) {
-            return curKey;
-        }
-    }
-    return 0xFFFF;
+    return ProgSettings::languageCodeToId(QString(langcode));
 }
 
-/* Converts back the language code to LCID */
+/* Converts the numeric language ID to a language code string */
 void
-plat_language_code_r(uint32_t lcid, char *outbuf, int len)
+plat_language_code_r(int id, char *outbuf, int len)
 {
-    if (!ProgSettings::lcid_langcode.contains(lcid)) {
-        qstrncpy(outbuf, "system", len);
-        return;
-    }
-    qstrncpy(outbuf, ProgSettings::lcid_langcode[lcid].first.toUtf8().constData(), len);
+    qstrncpy(outbuf, ProgSettings::languageIdToCode(id).toUtf8().constData(), len);
     return;
 }
 
@@ -545,7 +568,7 @@ endblit()
         std::this_thread::sleep_for(std::chrono::milliseconds(1));
     }
 }
-}
+} /*extern "C" */
 
 #ifdef Q_OS_WINDOWS
 size_t
@@ -592,12 +615,10 @@ c16stombs(char dst[], const uint16_t src[], int len)
 #        define LIB_NAME_GPCL "gpcl6dll32.dll"
 #    endif
 #    define LIB_NAME_PCAP        "Npcap"
-#    define MOUSE_CAPTURE_KEYSEQ "F8+F12"
 #else
 #    define LIB_NAME_GS          "libgs"
 #    define LIB_NAME_GPCL        "libgpcl6"
 #    define LIB_NAME_PCAP        "libpcap"
-#    define MOUSE_CAPTURE_KEYSEQ "Ctrl+End"
 #endif
 
 QMap<int, std::wstring> ProgSettings::translatedstrings;
@@ -607,8 +628,8 @@ ProgSettings::reloadStrings()
 {
     translatedstrings.clear();
     translatedstrings[STRING_MOUSE_CAPTURE]             = QCoreApplication::translate("", "Click to capture mouse").toStdWString();
-    translatedstrings[STRING_MOUSE_RELEASE]             = QCoreApplication::translate("", "Press %1 to release mouse").arg(QCoreApplication::translate("", MOUSE_CAPTURE_KEYSEQ)).toStdWString();
-    translatedstrings[STRING_MOUSE_RELEASE_MMB]         = QCoreApplication::translate("", "Press %1 or middle button to release mouse").arg(QCoreApplication::translate("", MOUSE_CAPTURE_KEYSEQ)).toStdWString();
+    translatedstrings[STRING_MOUSE_RELEASE]             = QCoreApplication::translate("", "Press %1 to release mouse").arg(QKeySequence(acc_keys[FindAccelerator("release_mouse")].seq, QKeySequence::PortableText).toString(QKeySequence::NativeText)).toStdWString();
+    translatedstrings[STRING_MOUSE_RELEASE_MMB]         = QCoreApplication::translate("", "Press %1 or middle button to release mouse").arg(QKeySequence(acc_keys[FindAccelerator("release_mouse")].seq, QKeySequence::PortableText).toString(QKeySequence::NativeText)).toStdWString();
     translatedstrings[STRING_INVALID_CONFIG]            = QCoreApplication::translate("", "Invalid configuration").toStdWString();
     translatedstrings[STRING_NO_ST506_ESDI_CDROM]       = QCoreApplication::translate("", "MFM/RLL or ESDI CD-ROM drives never existed").toStdWString();
     translatedstrings[STRING_PCAP_ERROR_NO_DEVICES]     = QCoreApplication::translate("", "No PCap devices found").toStdWString();
@@ -620,11 +641,14 @@ ProgSettings::reloadStrings()
     translatedstrings[STRING_GHOSTPCL_ERROR_DESC]       = QCoreApplication::translate("", "%1 is required for automatic conversion of PCL files to PDF.\n\nAny documents sent to the generic PCL printer will be saved as Printer Command Language (.pcl) files.").arg(LIB_NAME_GPCL).toStdWString();
     translatedstrings[STRING_HW_NOT_AVAILABLE_MACHINE]  = QCoreApplication::translate("", "Machine \"%hs\" is not available due to missing ROMs in the roms/machines directory. Switching to an available machine.").toStdWString();
     translatedstrings[STRING_HW_NOT_AVAILABLE_VIDEO]    = QCoreApplication::translate("", "Video card \"%hs\" is not available due to missing ROMs in the roms/video directory. Switching to an available video card.").toStdWString();
-    translatedstrings[STRING_HW_NOT_AVAILABLE_VIDEO2]   = QCoreApplication::translate("", "Video card #2 \"%hs\"  is not available due to missing ROMs in the roms/video directory. Disabling the second video card.").toStdWString();
+    translatedstrings[STRING_HW_NOT_AVAILABLE_VIDEO2]   = QCoreApplication::translate("", "Video card #2 \"%hs\" is not available due to missing ROMs in the roms/video directory. Disabling the second video card.").toStdWString();
+    translatedstrings[STRING_HW_NOT_AVAILABLE_DEVICE]   = QCoreApplication::translate("", "Device \"%hs\" is not available due to missing ROMs. Ignoring the device.").toStdWString();
     translatedstrings[STRING_HW_NOT_AVAILABLE_TITLE]    = QCoreApplication::translate("", "Hardware not available").toStdWString();
     translatedstrings[STRING_MONITOR_SLEEP]             = QCoreApplication::translate("", "Monitor in sleep mode").toStdWString();
     translatedstrings[STRING_NET_ERROR]                 = QCoreApplication::translate("", "Failed to initialize network driver").toStdWString();
     translatedstrings[STRING_NET_ERROR_DESC]            = QCoreApplication::translate("", "The network configuration will be switched to the null driver").toStdWString();
+    translatedstrings[STRING_ESCP_ERROR_TITLE]          = QCoreApplication::translate("", "Unable to find Dot-Matrix fonts").toStdWString();
+    translatedstrings[STRING_ESCP_ERROR_DESC]           = QCoreApplication::translate("", "TrueType fonts in the \"roms/printer/fonts\" directory are required for the emulation of the Generic ESC/P Dot-Matrix Printer.").toStdWString();
 }
 
 wchar_t *
@@ -642,9 +666,9 @@ plat_chdir(char *path)
 }
 
 void
-plat_get_global_config_dir(char *outbuf, const uint8_t len)
+plat_get_global_config_dir(char *outbuf, const size_t len)
 {
-    const auto dir = QDir(QStandardPaths::standardLocations(QStandardPaths::AppConfigLocation)[0]);
+    const auto dir = QDir(QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation));
     if (!dir.exists()) {
         if (!dir.mkpath(".")) {
             qWarning("Failed to create global configuration directory %s", dir.absolutePath().toUtf8().constData());
@@ -654,9 +678,9 @@ plat_get_global_config_dir(char *outbuf, const uint8_t len)
 }
 
 void
-plat_get_global_data_dir(char *outbuf, const uint8_t len)
+plat_get_global_data_dir(char *outbuf, const size_t len)
 {
-    const auto dir = QDir(QStandardPaths::standardLocations(QStandardPaths::AppDataLocation)[0]);
+    const auto dir = QDir(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation));
     if (!dir.exists()) {
         if (!dir.mkpath(".")) {
             qWarning("Failed to create global data directory %s", dir.absolutePath().toUtf8().constData());
@@ -668,10 +692,21 @@ plat_get_global_data_dir(char *outbuf, const uint8_t len)
 void
 plat_get_temp_dir(char *outbuf, const uint8_t len)
 {
-    const auto dir = QDir(QStandardPaths::standardLocations(QStandardPaths::TempLocation)[0]);
+    const auto dir = QDir(QStandardPaths::writableLocation(QStandardPaths::TempLocation));
     strncpy(outbuf, dir.canonicalPath().toUtf8().constData(), len);
 }
 
+void
+plat_get_vmm_dir(char *outbuf, const size_t len)
+{
+#ifdef Q_OS_WINDOWS
+    const auto path = QDir::home().filePath("86Box VMs");
+#else
+    const auto path = QDir(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation)).filePath("Virtual Machines");
+#endif
+    strncpy(outbuf, path.toUtf8().constData(), len);
+}
+
 void
 plat_init_rom_paths(void)
 {
@@ -786,8 +821,8 @@ plat_set_thread_name(void *thread, const char *name)
 
     if (pSetThreadDescription) {
         size_t len = strlen(name) + 1;
-        wchar_t wname[len + 1];
-        mbstowcs(wname, name, len);
+        wchar_t wname[2048];
+        mbstowcs(wname, name, (len >= 1024) ? 1024 : len);
         pSetThreadDescription(thread ? (HANDLE) thread : GetCurrentThread(), wname);
     }
 #else
@@ -795,14 +830,32 @@ plat_set_thread_name(void *thread, const char *name)
     if (thread) /* Apple pthread can only set self's name */
         return;
     char truncated[64];
+#    elif defined(Q_OS_NETBSD)
+    char truncated[64];
 #    else
     char truncated[16];
 #    endif
     strncpy(truncated, name, sizeof(truncated) - 1);
-#    ifdef Q_OS_DARWIN
+#    if defined(Q_OS_DARWIN)
     pthread_setname_np(truncated);
+#    elif defined(Q_OS_NETBSD)
+    pthread_setname_np(thread ? *((pthread_t *) thread) : pthread_self(), truncated, (void*)"%s");
+#    elif defined(__HAIKU__)
+    rename_thread(find_thread(NULL), truncated);
+#    elif defined(Q_OS_OPENBSD)
+    pthread_set_name_np(thread ? *((pthread_t *) thread) : pthread_self(), truncated);
 #    else
     pthread_setname_np(thread ? *((pthread_t *) thread) : pthread_self(), truncated);
 #    endif
 #endif
 }
+
+void
+plat_break(void)
+{
+#ifdef Q_OS_WINDOWS
+    DebugBreak();
+#else
+    raise(SIGTRAP);
+#endif
+}
