@@ -22,18 +22,23 @@ public slots:
     void onCurrentMachineChanged(int machineId);
 
 private slots:
-    void on_pushButtonConfigureSecondary_clicked();
-
-private slots:
-    void on_comboBoxVideoSecondary_currentIndexChanged(int index);
-
-private slots:
-    void on_checkBoxVoodoo_stateChanged(int state);
-    void on_checkBoxXga_stateChanged(int state);
     void on_comboBoxVideo_currentIndexChanged(int index);
+    void on_pushButtonConfigureVideo_clicked();
+
+    void on_comboBoxVideoSecondary_currentIndexChanged(int index);
+    void on_pushButtonConfigureVideoSecondary_clicked();
+
+    void on_checkBoxVoodoo_stateChanged(int state);
     void on_pushButtonConfigureVoodoo_clicked();
+
+    void on_checkBox8514_stateChanged(int state);
+    void on_pushButtonConfigure8514_clicked();
+
+    void on_checkBoxXga_stateChanged(int state);
     void on_pushButtonConfigureXga_clicked();
-    void on_pushButtonConfigure_clicked();
+
+    void on_checkBoxDa2_stateChanged(int state);
+    void on_pushButtonConfigureDa2_clicked();
 
 private:
     Ui::SettingsDisplay *ui;
