@@ -32,6 +32,7 @@
 #include <86box/fdd.h>
 #include <86box/fdc.h>
 #include <86box/gameport.h>
+#include <86box/keyboard.h>
 #include <86box/sio.h>
 #include <86box/isapnp.h>
 #include <86box/plat_fallthrough.h>
@@ -243,6 +244,7 @@ typedef struct it86x1f_t {
 
     fdc_t    *fdc;
     serial_t *uart[2];
+    lpt_t    *lpt;
     void     *gameport;
 } it86x1f_t;
 
@@ -290,11 +292,14 @@ it8661f_pnp_config_changed(uint8_t ld, isapnp_device_config_t *config, void *pri
             break;
 
         case 3:
-            lpt1_remove();
+            lpt_port_remove(dev->lpt);
 
             if (config->activate && (config->io[0].base != ISAPNP_IO_DISABLED)) {
                 it86x1f_log("IT86x1F: LPT enabled at port %04X IRQ %d\n", config->io[0].base, config->irq[0].irq);
-                lpt1_init(config->io[0].base);
+                lpt_port_setup(dev->lpt, config->io[0].base);
+
+                lpt_port_irq(dev->lpt, config->irq[0].irq);
+                lpt_port_dma(dev->lpt, (config->dma[0].dma == ISAPNP_DMA_DISABLED) ? -1 : config->dma[0].dma);
             } else {
                 it86x1f_log("IT86x1F: LPT disabled\n");
             }
@@ -465,6 +470,13 @@ it86x1f_pnp_write_vendor_reg(uint8_t ld, uint8_t reg, uint8_t val, void *priv)
                 case 0x0f0:
                     dev->ldn_regs[ld][reg & 0x0f] = val & 0x0f;
                     fdc_set_swwp(dev->fdc, !!(val & 0x01));
+                    if (val & 0x02) {
+                        for (int i = 0; i < 4; i++)
+                            fdc_update_drvrate(dev->fdc, i, 1);
+                    } else {
+                        for (int i = 0; i < 4; i++)
+                            fdc_update_drvrate(dev->fdc, i, 0);
+                    }
                     fdc_set_swap(dev->fdc, !!(val & 0x04));
                     break;
 
@@ -483,6 +495,8 @@ it86x1f_pnp_write_vendor_reg(uint8_t ld, uint8_t reg, uint8_t val, void *priv)
 
                 case 0x3f0:
                     dev->ldn_regs[ld][reg & 0x0f] = val & 0x07;
+                    lpt_set_epp(dev->lpt, val & 0x01);
+                    lpt_set_ecp(dev->lpt, val & 0x02);
                     break;
 
                 case 0x4f0:
@@ -602,7 +616,7 @@ it86x1f_pnp_write_vendor_reg(uint8_t ld, uint8_t reg, uint8_t val, void *priv)
 }
 
 static void
-it86x1f_write_addr(uint16_t port, uint8_t val, void *priv)
+it86x1f_write_addr(UNUSED(uint16_t port), uint8_t val, void *priv)
 {
     it86x1f_t *dev = (it86x1f_t *) priv;
 
@@ -623,7 +637,7 @@ it86x1f_write_addr(uint16_t port, uint8_t val, void *priv)
 }
 
 static void
-it86x1f_write_data(uint16_t port, uint8_t val, void *priv)
+it86x1f_write_data(UNUSED(uint16_t port), uint8_t val, void *priv)
 {
     it86x1f_t *dev = (it86x1f_t *) priv;
 
@@ -659,7 +673,7 @@ it86x1f_write_data(uint16_t port, uint8_t val, void *priv)
 }
 
 static uint8_t
-it86x1f_read_addr(uint16_t port, void *priv)
+it86x1f_read_addr(UNUSED(uint16_t port), void *priv)
 {
     it86x1f_t *dev = (it86x1f_t *) priv;
     uint8_t    ret = dev->locked ? 0xff : dev->cur_reg;
@@ -670,7 +684,7 @@ it86x1f_read_addr(uint16_t port, void *priv)
 }
 
 static uint8_t
-it86x1f_read_data(uint16_t port, void *priv)
+it86x1f_read_data(UNUSED(uint16_t port), void *priv)
 {
     it86x1f_t *dev = (it86x1f_t *) priv;
     uint8_t    ret = 0xff;
@@ -771,13 +785,19 @@ it86x1f_reset(it86x1f_t *dev)
 {
     it86x1f_log("IT86x1F: reset()\n");
 
+    for (int i = 0; i < 4; i++)
+        fdc_update_drvrate(dev->fdc, i, 0);
+
     fdc_reset(dev->fdc);
 
     serial_remove(dev->uart[0]);
 
     serial_remove(dev->uart[1]);
 
-    lpt1_remove();
+    lpt_port_remove(dev->lpt);
+
+    lpt_set_epp(dev->lpt, 0);
+    lpt_set_ecp(dev->lpt, 0);
 
     isapnp_enable_card(dev->pnp_card, ISAPNP_CARD_DISABLE);
 
@@ -799,8 +819,7 @@ it86x1f_close(void *priv)
 static void *
 it86x1f_init(UNUSED(const device_t *info))
 {
-    it86x1f_t *dev = (it86x1f_t *) malloc(sizeof(it86x1f_t));
-    memset(dev, 0, sizeof(it86x1f_t));
+    it86x1f_t *dev = (it86x1f_t *) calloc(1, sizeof(it86x1f_t));
 
     uint8_t i;
     for (i = 0; i < (sizeof(it86x1f_models) / sizeof(it86x1f_models[0])); i++) {
@@ -823,6 +842,11 @@ it86x1f_init(UNUSED(const device_t *info))
     dev->uart[0] = device_add_inst(&ns16550_device, 1);
     dev->uart[1] = device_add_inst(&ns16550_device, 2);
 
+    dev->lpt = device_add_inst(&lpt_port_device, 1);
+
+    lpt_set_cnfgb_readout(dev->lpt, 0x00);
+    lpt_set_ext(dev->lpt, 1);
+
     dev->gameport = gameport_add(&gameport_sio_device);
 
     dev->instance = device_get_instance();
@@ -831,6 +855,9 @@ it86x1f_init(UNUSED(const device_t *info))
     dev->unlock_id = it86x1f_models[i].unlock_id;
     io_sethandler(0x279, 1, NULL, NULL, NULL, it86x1f_write_unlock, NULL, NULL, dev);
 
+    if (info->local == ITE_IT8671F)
+        device_add_params(&kbc_at_device, (void *) (KBC_VEN_AMI | 0x00004800));
+
     it86x1f_reset(dev);
 
     return dev;
@@ -844,7 +871,7 @@ const device_t it8661f_device = {
     .init          = it86x1f_init,
     .close         = it86x1f_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
@@ -858,7 +885,7 @@ const device_t it8671f_device = {
     .init          = it86x1f_init,
     .close         = it86x1f_close,
     .reset         = NULL,
-    { .available = NULL },
+    .available     = NULL,
     .speed_changed = NULL,
     .force_redraw  = NULL,
     .config        = NULL
